version: '3.6'
services:
    {{ opts.service_name }}:
        image: python:{{ aux.project.minimal_version }}-slim-bullseye
        volumes:
            - {{ aux.payload.docker_settings.project_dir }}:/home/container/workdir:rw
        working_dir: /home/container/workdir
        command: [
            /bin/sh, -c,
            "pip install twine && python setup.py sdist && TWINE_PASSWORD=${PYPI_PASSWORD:-} TWINE_USERNAME=${PYPI_USERNAME:-} python -m twine upload dist/*"
        ]
